@@ -84,16 +84,17 @@ def getPrettyFromXml(xmlstr):
 
 __all__.append("getPrettyFromXml")
 
+RE_XML_NAMESPACE = re.compile(b'^(<?[^>]+?>\s*|\s*)(<\w+) xmlns=[\'"](http://[^\'"]+)[\'"]', re.MULTILINE)
 
 def stripNameSpace(xml):
     """
     removeNameSpace(xml) -- remove top-level AWS namespace
     Operate on raw byte(utf-8) xml string. (Not unicode)
     """
-    r = re.compile(b'^(<?[^>]+?>\s*)(<\w+) xmlns=[\'"](http://[^\'"]+)[\'"](.*)', re.MULTILINE)
-    if r.match(xml):
-        xmlns = r.match(xml).groups()[2]
-        xml = r.sub("\\1\\2\\4", xml)
+    xmlns_match = RE_XML_NAMESPACE.match(xml)
+    if xmlns_match:
+        xmlns = xmlns_match.group(3)
+        xml = RE_XML_NAMESPACE.sub("\\1\\2", xml, 1)
     else:
         xmlns = None
     return xml, xmlns
@@ -169,9 +170,11 @@ def appendXmlTextNode(tag_name, text, parent):
     return el
 __all__.append("appendXmlTextNode")
 
+RE_S3_DATESTRING = re.compile('\.[0-9]*(?:[Z\\-\\+]*?)')
+
 def dateS3toPython(date):
     # Reset milliseconds to 000
-    date = re.compile('\.[0-9]*(?:[Z\\-\\+]*?)').sub(".000", date)
+    date = RE_S3_DATESTRING.sub(".000", date)
     return dateutil.parser.parse(date, fuzzy=True)
 __all__.append("dateS3toPython")
 
@@ -255,14 +258,13 @@ __all__.append("mktmpfile")
 
 def hash_file_md5(filename):
     h = md5()
-    f = open(deunicodise(filename), "rb")
-    while True:
-        # Hash 32kB chunks
-        data = f.read(32*1024)
-        if not data:
-            break
-        h.update(data)
-    f.close()
+    with open(deunicodise(filename), "rb") as fp:
+        while True:
+            # Hash 32kB chunks
+            data = fp.read(32*1024)
+            if not data:
+                break
+            h.update(data)
     return h.hexdigest()
 __all__.append("hash_file_md5")
 
@@ -520,7 +522,7 @@ def getBucketFromHostname(hostname):
         return (hostname, False)
 
     # Create RE pattern from Config.host_bucket
-    pattern = S3.Config.Config().host_bucket % { 'bucket' : '(?P<bucket>.*)' }
+    pattern = S3.Config.Config().host_bucket.lower() % { 'bucket' : '(?P<bucket>.*)' }
     m = re.match(pattern, hostname, re.UNICODE)
     if not m:
         return (hostname, False)
@@ -528,7 +530,7 @@ def getBucketFromHostname(hostname):
 __all__.append("getBucketFromHostname")
 
 def getHostnameFromBucket(bucket):
-    return S3.Config.Config().host_bucket % { 'bucket' : bucket }
+    return S3.Config.Config().host_bucket.lower() % { 'bucket' : bucket }
 __all__.append("getHostnameFromBucket")
 
 
@@ -545,8 +547,6 @@ def calculateChecksum(buffer, mfile, offset, chunk_size, send_chunk):
         md5_hash.update(buffer)
 
     return md5_hash.hexdigest()
-
-
 __all__.append("calculateChecksum")
 
 
